// PlotArea.cpp: Implementierungsdatei
//

#include "stdafx.h"
#include "Plotter.h"
#include "PlotArea.h"

#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPlotArea

CPlotArea::CPlotArea()
{
	m_formula.setExpression("(sin(x)/((x^2)+20))*200");
}

CPlotArea::~CPlotArea()
{
}


LRESULT CALLBACK EXPORT
CPlotArea::WndProcHook(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	// create new item and attach it
	CPlotArea* pPlotArea = new CPlotArea();
	pPlotArea->Attach(hWnd);

	// set up wndproc to AFX one, and call it
	::SetWindowLong(hWnd, GWL_WNDPROC, (DWORD)AfxWndProc);

	return ::CallWindowProc(AfxWndProc, hWnd, msg, wParam, lParam);
}


void CPlotArea::PostNcDestroy()
{
	// needed to clean up the C++ CWnd object
	delete this;
}


BOOL CPlotArea::RegisterControlClass()
{
	WNDCLASS wcls;

	// check to see if class already registered
	static const TCHAR szClass[] = _T("plotarea");
	if (::GetClassInfo(AfxGetInstanceHandle(), szClass, &wcls))
	{
		// name already registered - ok if it was us
		return (wcls.lpfnWndProc == (WNDPROC)CPlotArea::WndProcHook);
	}

	// set new values
	wcls.lpfnWndProc = CPlotArea::WndProcHook;
	wcls.hInstance = AfxGetInstanceHandle();
	wcls.lpszClassName = szClass;
	wcls.style            = CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
	wcls.cbClsExtra       = wcls.cbWndExtra = 0;
	wcls.hIcon            = NULL;
	wcls.hCursor          = NULL;
	wcls.hbrBackground    = (HBRUSH) GetStockObject (WHITE_BRUSH);
	wcls.lpszMenuName     = NULL;


	return (RegisterClass(&wcls) != 0);
}

BEGIN_MESSAGE_MAP(CPlotArea, CWnd)
	//{{AFX_MSG_MAP(CPlotArea)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// Behandlungsroutinen fr Nachrichten CPlotArea 


void CPlotArea::OnPaint() 
{
	CPaintDC dc(this);
	dc.SetBkMode(TRANSPARENT);

	// bounding
	double x0 = theApp.getX0();
	double y0 = theApp.getY0();
	double x1 = theApp.getX1();
	double y1 = theApp.getY1();

	if (x0 < x1 && y0 < y1)
	{
		// calculate scaling
		RECT size;
		GetClientRect(&size);
		double w = size.right;
		double h = size.bottom;
		double mx = (x1 - x0) / double(w);
		double my = (y1 - y0) / double(h);

		// create pens and select first pen
		CPen gridPen(PS_SOLID, 1, RGB(192, 192, 192));
		CPen axisPen(PS_SOLID, 2, RGB(0, 0, 0));
		CPen functionPen(PS_SOLID, 3, RGB(255, 32, 32));
		CPen textPen(PS_SOLID, 1, RGB(0, 0, 0));
		CPen* pPenOld = dc.SelectObject(&gridPen);

		// draw grid
		double steps = 10;
		double start = 0;
		double tens;
		if (x0 != 0) {
			tens = pow(10, floor(log10(fabs(x0))));
			start = floor(x0 / tens) * tens;
		}
		double step = (x1 - start) / steps;
		tens = pow(10, floor(log10(fabs(step))));
		step = floor(step / tens) * tens;
		for (int s = 0; s < steps * 2; s++) {
			int x = int((start + double(s) * step - x0) / mx + 0.5);
			dc.MoveTo(x, 0);
			dc.LineTo(x, h);
		}
		steps = 7;
		start = 0;
		if (y0 != 0) {
			tens = pow(10, floor(log10(fabs(y0))));
			start = floor(y0 / tens) * tens;
		}
		step = (y1 - start) / steps;
		tens = pow(10, floor(log10(fabs(step))));
		step = floor(step / tens) * tens;
		for (s = 0; s < steps * 2; s++) {
			int y = int((start + double(s) * step - y0) / my + 0.5);
			dc.MoveTo(0, h - y);
			dc.LineTo(w, h - y);
		}

		// draw axis
		dc.SelectObject(&axisPen);
		dc.MoveTo(0, h - int(-y0 / my + 0.5));
		dc.LineTo(w, h - int(-y0 / my + 0.5));
		dc.MoveTo(-x0 / mx + 0.5, 0);
		dc.LineTo(-x0 / mx + 0.5, h);
		if (pPenOld != NULL) dc.SelectObject(pPenOld);

		// draw function
		bool moveTo = true;
		dc.SelectObject(&functionPen);
		try {
			m_formula.setExpression(theApp.getFunction());
			for (int i = 0; i < w; i++) {
				try {
					double x = double(i) * mx + x0;
					m_formula.setVariable("x", x);
					double y = m_formula.eval();
					int yc = h - int((y - y0) / my + 0.5);
					if (moveTo) {
						dc.MoveTo(i, yc);
						moveTo = false;
					} else {
						dc.LineTo(i, yc);
					}
				} catch (ParserException&) {
					moveTo = true;
				}
			}
		} catch (ParserException&) {
		}

		// draw text
		dc.SelectObject(&textPen);
		steps = 10;
		start = 0;
		tens;
		if (x0 != 0) {
			tens = pow(10, floor(log10(fabs(x0))));
			start = floor(x0 / tens) * tens;
		}
		step = (x1 - start) / steps;
		tens = pow(10, floor(log10(fabs(step))));
		step = floor(step / tens) * tens;
		for (s = 0; s < steps * 2; s++) {
			CString text;
			text.Format("%g", start + double(s) * step);
			int x = int((start + double(s) * step - x0) / mx + 0.5);
			dc.TextOut(x, h - int(-y0 / my + 0.5), text);
		}
		steps = 7;
		start = 0;
		if (y0 != 0) {
			tens = pow(10, floor(log10(fabs(y0))));
			start = floor(y0 / tens) * tens;
		}
		step = (y1 - start) / steps;
		tens = pow(10, floor(log10(fabs(step))));
		step = floor(step / tens) * tens;
		for (s = 0; s < steps * 2; s++) {
			CString text;
			text.Format("%g", start + double(s) * step);
			int y = int((start + double(s) * step - y0) / my + 0.5);
			dc.TextOut(-x0 / mx + 0.5, h - y, text);
		}

		// restore old pen
		if (pPenOld != NULL) dc.SelectObject(pPenOld);
	}
}
